/**
 * File:
 *  main.c
 *
 * Notes:
 *  This application demostrates the modbus slave implementation
 *  Uses UART1 routines for communicating with serial RS232 port
 *
 * History:
 *  10/06/2012 Written by Simon Platten
 *  30/06/2012 Added mapping parameters to registers to allow 8 bit data ports
 *             to be mapped into 16 bit registers, also changed the number
 *             of holding registers to 2 and the number of input registers to 3.
 *             Changed I/O type array sizes to:
 *                     coils 20
 *                     status bits 28
 *                     holding registers 2
 *                     input registers 2
 */
#include "modbusSlave.h"

static statusDef statusBits[28];
static holdingDef holdingReg[2];
static inputDef inputReg[2];
static coilDef coils[20];
/**
 * Interrupt Service Routine
 */
void interrupt() {
  decodePacket();
}
/**
 * Entry point
 */
void main() {
  int p, i;
#ifdef P18F45K22
  ANSELA = 0;
  ANSELB = 0;
  ANSELC = 0;
  ANSELD = 0;
  ANSELE = 0;
#endif
#ifdef P18F4550
  ADCON1 = 0x0f;                 // Configure all ports as digital
#endif
// Configure PORTA, PORTB, PORTC and PORTE as output
  TRISA = 0;
  TRISB = 0;
  TRISC = 0;
  TRISE = 0;
// Configure PORTD as input
  TRISD = 0xff;
// Initialise ports
  LATA = 0;
  LATB = 0;
  LATC = 0;
  LATE = 0;
// Make sure all the modbus type arrays are cleared
  memset(statusBits, 0, sizeof(statusBits));
  memset(holdingReg, 0, sizeof(holdingReg));
  memset(inputReg, 0, sizeof(inputReg));
  memset(coils, 0, sizeof(coils));
// Create an instance of the modbus slave
  modbusSlave(BAUD_115200, 1, 1);
  i = 0;
  for( p=0; p<8; p++, i++ ) {
    addCoil(&coils[i], i + 1, &LATB, p);
  }
  for( p=0; p<8; p++, i++ ) {
    addCoil(&coils[i], i + 1, &LATC, p);
  }
  for( p=0; p<4; p++, i++ ) {
    addCoil(&coils[i], i + 1, &LATE, p);
  }
  i = 0;
  for( p=0; p<8; p++, i++ ) {
    addStatusBit(&statusBits[i], i + 1, &PORTB, p);
  }
  for( p=0; p<8; p++, i++ ) {
    addStatusBit(&statusBits[i], i + 1, &PORTC, p);
  }
  for( p=0; p<8; p++, i++ ) {
    addStatusBit(&statusBits[i], i + 1, &PORTD, p);
  }
  for( p=0; p<4; p++, i++ ) {
    addStatusBit(&statusBits[i], i + 1, &PORTE, p);
  }
  addInputReg(&inputReg[0], 1, 0, (uint*)&PORTB);
  addInputReg(&inputReg[0], 1, 1, (uint*)&PORTC);
  addInputReg(&inputReg[1], 2, 0, (uint*)&PORTD);
  addInputReg(&inputReg[1], 2, 1, (uint*)&PORTE);
  addHoldingReg(&holdingReg[0], 1, 0, (uint*)&PORTB);
  addHoldingReg(&holdingReg[0], 1, 1, (uint*)&PORTC);
  addHoldingReg(&holdingReg[1], 2, 0, (uint*)&PORTE);
// Endless loop
  while (1) {
  }
}